Struct BoxedStream
pub struct BoxedStream<T, E> {
pub(crate) stream: Pin<Box<dyn Stream<Item = Result<T, E>> + Send>>,
}Expand description
A boxed stream type that can be used with the websocket protocol.
You can easily convert any static type that implement [futures::Stream] into a BoxedStream
with the From trait.
Example
use futures::StreamExt;
use server_fn::{BoxedStream, ServerFnError};
let stream: BoxedStream<_, ServerFnError> =
futures::stream::iter(0..10).map(Result::Ok).into();Fields
stream: Pin<Box<dyn Stream<Item = Result<T, E>> + Send>>Methods from Deref<Target = Pin<Box<dyn Stream<Item = Result<T, E>> + Send>>>
1.33.0 ยท Sourcepub fn as_ref(&self) -> Pin<&<Ptr as Deref>::Target>where
Ptr: Deref,
pub fn as_ref(&self) -> Pin<&<Ptr as Deref>::Target>where
Ptr: Deref,
Gets a shared reference to the pinned value this Pin points to.
This is a generic method to go from &Pin<Pointer<T>> to Pin<&T>.
It is safe because, as part of the contract of Pin::new_unchecked,
the pointee cannot move after Pin<Pointer<T>> got created.
โMaliciousโ implementations of Pointer::Deref are likewise
ruled out by the contract of Pin::new_unchecked.
1.33.0 ยท Sourcepub fn as_mut(&mut self) -> Pin<&mut <Ptr as Deref>::Target>where
Ptr: DerefMut,
pub fn as_mut(&mut self) -> Pin<&mut <Ptr as Deref>::Target>where
Ptr: DerefMut,
Gets a mutable reference to the pinned value this Pin<Ptr> points to.
This is a generic method to go from &mut Pin<Pointer<T>> to Pin<&mut T>.
It is safe because, as part of the contract of Pin::new_unchecked,
the pointee cannot move after Pin<Pointer<T>> got created.
โMaliciousโ implementations of Pointer::DerefMut are likewise
ruled out by the contract of Pin::new_unchecked.
This method is useful when doing multiple calls to functions that consume the pinning pointer.
Example
use std::pin::Pin;
impl Type {
fn method(self: Pin<&mut Self>) {
// do something
}
fn call_method_twice(mut self: Pin<&mut Self>) {
// `method` consumes `self`, so reborrow the `Pin<&mut Self>` via `as_mut`.
self.as_mut().method();
self.as_mut().method();
}
}1.84.0 ยท Sourcepub fn as_deref_mut(
self: Pin<&mut Pin<Ptr>>,
) -> Pin<&mut <Ptr as Deref>::Target>where
Ptr: DerefMut,
pub fn as_deref_mut(
self: Pin<&mut Pin<Ptr>>,
) -> Pin<&mut <Ptr as Deref>::Target>where
Ptr: DerefMut,
Gets Pin<&mut T> to the underlying pinned value from this nested Pin-pointer.
This is a generic method to go from Pin<&mut Pin<Pointer<T>>> to Pin<&mut T>. It is
safe because the existence of a Pin<Pointer<T>> ensures that the pointee, T, cannot
move in the future, and this method does not enable the pointee to move. โMaliciousโ
implementations of Ptr::DerefMut are likewise ruled out by the contract of
Pin::new_unchecked.
1.33.0 ยท Sourcepub fn set(&mut self, value: <Ptr as Deref>::Target)
pub fn set(&mut self, value: <Ptr as Deref>::Target)
Assigns a new value to the memory location pointed to by the Pin<Ptr>.
This overwrites pinned data, but that is okay: the original pinned valueโs destructor gets
run before being overwritten and the new value is also a valid value of the same type, so
no pinning invariant is violated. See the pin module documentation
for more information on how this upholds the pinning invariants.
Example
use std::pin::Pin;
let mut val: u8 = 5;
let mut pinned: Pin<&mut u8> = Pin::new(&mut val);
println!("{}", pinned); // 5
pinned.set(10);
println!("{}", pinned); // 10Trait Implementations
impl<T, E> Debug for BoxedStream<T, E>
impl<T, E> Debug for BoxedStream<T, E>
impl<T, E> Deref for BoxedStream<T, E>
impl<T, E> Deref for BoxedStream<T, E>
impl<T, E> DerefMut for BoxedStream<T, E>
impl<T, E> DerefMut for BoxedStream<T, E>
fn deref_mut(&mut self) -> &mut <BoxedStream<T, E> as Deref>::Target
fn deref_mut(&mut self) -> &mut <BoxedStream<T, E> as Deref>::Target
impl<T, E, S> From<S> for BoxedStream<T, E>
impl<T, E, S> From<S> for BoxedStream<T, E>
fn from(stream: S) -> BoxedStream<T, E>
fn from(stream: S) -> BoxedStream<T, E>
impl<Input, InputItem, OutputItem, InputEncoding, OutputEncoding, Client, Server, Error, InputStreamError, OutputStreamError> Protocol<Input, BoxedStream<OutputItem, OutputStreamError>, Client, Server, Error, InputStreamError, OutputStreamError> for Websocket<InputEncoding, OutputEncoding>where
Input: Deref<Target = BoxedStream<InputItem, InputStreamError>> + Into<BoxedStream<InputItem, InputStreamError>> + From<BoxedStream<InputItem, InputStreamError>>,
InputEncoding: Encodes<InputItem> + Decodes<InputItem>,
OutputEncoding: Encodes<OutputItem> + Decodes<OutputItem>,
InputStreamError: FromServerFnError + Send,
OutputStreamError: FromServerFnError + Send,
Error: FromServerFnError + Send,
Server: Server<Error, InputStreamError, OutputStreamError>,
Client: Client<Error, InputStreamError, OutputStreamError>,
OutputItem: Send + 'static,
InputItem: Send + 'static,
impl<Input, InputItem, OutputItem, InputEncoding, OutputEncoding, Client, Server, Error, InputStreamError, OutputStreamError> Protocol<Input, BoxedStream<OutputItem, OutputStreamError>, Client, Server, Error, InputStreamError, OutputStreamError> for Websocket<InputEncoding, OutputEncoding>where
Input: Deref<Target = BoxedStream<InputItem, InputStreamError>> + Into<BoxedStream<InputItem, InputStreamError>> + From<BoxedStream<InputItem, InputStreamError>>,
InputEncoding: Encodes<InputItem> + Decodes<InputItem>,
OutputEncoding: Encodes<OutputItem> + Decodes<OutputItem>,
InputStreamError: FromServerFnError + Send,
OutputStreamError: FromServerFnError + Send,
Error: FromServerFnError + Send,
Server: Server<Error, InputStreamError, OutputStreamError>,
Client: Client<Error, InputStreamError, OutputStreamError>,
OutputItem: Send + 'static,
InputItem: Send + 'static,
async fn run_server<F, Fut>(
request: <Server as Server<Error, InputStreamError, OutputStreamError>>::Request,
server_fn: F,
) -> Result<<Server as Server<Error, InputStreamError, OutputStreamError>>::Response, Error>
async fn run_server<F, Fut>( request: <Server as Server<Error, InputStreamError, OutputStreamError>>::Request, server_fn: F, ) -> Result<<Server as Server<Error, InputStreamError, OutputStreamError>>::Response, Error>
fn run_client(
path: &str,
input: Input,
) -> impl Future<Output = Result<BoxedStream<OutputItem, OutputStreamError>, Error>> + Send
fn run_client( path: &str, input: Input, ) -> impl Future<Output = Result<BoxedStream<OutputItem, OutputStreamError>, Error>> + Send