ServerFnTraitObj

Struct ServerFnTraitObj 

pub struct ServerFnTraitObj<Req, Res> {
    pub(crate) path: &'static str,
    pub(crate) method: Method,
    pub(crate) handler: fn(Req) -> Pin<Box<dyn Future<Output = Res> + Send>>,
    pub(crate) middleware: fn() -> Vec<Arc<dyn Layer<Req, Res>>>,
    pub(crate) ser: fn(ServerFnErrorErr) -> Bytes,
}
A trait object that allows multiple server functions that take the same request type and return the same response type to be gathered into a single collection.

Fields

path: &'static strmethod: Methodhandler: fn(Req) -> Pin<Box<dyn Future<Output = Res> + Send>>middleware: fn() -> Vec<Arc<dyn Layer<Req, Res>>>ser: fn(ServerFnErrorErr) -> Bytes

Implementations



impl<Req, Res> ServerFnTraitObj<Req, Res>

pub const fn new<S>( handler: fn(Req) -> Pin<Box<dyn Future<Output = Res> + Send>>, ) -> ServerFnTraitObj<Req, Res>
where S: ServerFn, <S as ServerFn>::Server: Server<<S as ServerFn>::Error, <S as ServerFn>::InputStreamError, <S as ServerFn>::OutputStreamError, Request = Req, Response = Res>, Req: Req<<S as ServerFn>::Error, <S as ServerFn>::InputStreamError, <S as ServerFn>::OutputStreamError, WebsocketResponse = Res> + Send + 'static, Res: TryRes<<S as ServerFn>::Error> + Send + 'static,

Converts the relevant parts of a server function into a trait object.

pub fn path(&self) -> &'static str

The path of the server function.

pub fn method(&self) -> Method

The HTTP method the server function expects.

pub fn handler(&self, req: Req) -> impl Future<Output = Res> + Send

The handler for this server function.

pub fn middleware(&self) -> Vec<Arc<dyn Layer<Req, Res>>>

The set of middleware that should be applied to this function.

pub fn boxed(self) -> BoxedService<Req, Res>
where ServerFnTraitObj<Req, Res>: Service<Req, Res>, Req: 'static, Res: 'static,

Converts the server function into a boxed service.